'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.ipyKernelTryObservable = ipyKernelTryObservable;
exports.condaInfoObservable = condaInfoObservable;
exports.condaEnvsObservable = condaEnvsObservable;
exports.createKernelSpecsFromEnvs = createKernelSpecsFromEnvs;
exports.condaKernelsObservable = condaKernelsObservable;

var _Rx = require('rxjs/Rx');

var _Rx2 = _interopRequireDefault(_Rx);

var _spawnRx = require('spawn-rx');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const path = require('path');

// TODO: Check for sys.prefix/share/jupyter/kernels/*/kernel.json
/**
 * ipyKernelTryObservable check for existence of ipykernel in an env
 * @param  {Object} env [description]
 * @return {[type]}     [description]
 */
function ipyKernelTryObservable(env) {
  const executable = path.join(env.prefix, 'bin', 'python');
  return (0, _spawnRx.spawn)(executable, ['-m', 'ipykernel', '--version'], { split: true }).filter(x => x.source && x.source === 'stdout').mapTo(env).catch(err => _Rx2.default.Observable.empty());
}

function condaInfoObservable() {
  return (0, _spawnRx.spawn)('conda', ['info', '--json']).map(info => JSON.parse(info));
}

function condaEnvsObservable(condaInfo$) {
  return condaInfo$.map(info => {
    const envs = info.envs.map(env => ({ name: path.basename(env), prefix: env }));
    envs.push({ name: 'root', prefix: info.root_prefix });
    return envs;
  }).map(envs => envs.map(ipyKernelTryObservable)).mergeAll().mergeAll().toArray();
}

function createKernelSpecsFromEnvs(envs) {
  const displayPrefix = 'Python'; // Or R
  const languageKey = 'py'; // or r

  // TODO: Handle Windows & Conda
  const languageExe = 'bin/python';
  const jupyterBin = 'bin/jupyter';

  const langEnvs = {};

  for (const env of envs) {
    const base = env.prefix;
    const exePath = path.join(base, languageExe);
    const envName = env.name;
    const name = `conda-env-${ envName }-${ languageKey }`;
    langEnvs[name] = {
      display_name: `${ displayPrefix } [conda env:${ envName }]`,
      // TODO: Support default R kernel
      argv: [exePath, '-m', 'ipykernel', '-f', '{connection_file}'],
      language: 'python'
    };
  }
  return langEnvs;
}

function condaKernelsObservable() {
  return condaEnvsObservable(condaInfoObservable()).map(createKernelSpecsFromEnvs);
}